@@ -23,7 +23,7 @@ UPSAMPLE_ERROR=(
     'upsample called with scale==1. '
     'Use allow_identity=True to force.'
 )
-
+EPS=1e-8
 #
 # HELPERS
 #
@@ -323,6 +323,8 @@ class Stack(keras.Model):
             squeeze_excitation=False,
             squeeze_excitation_ratio=16,
             channel_squeeze_excitation=False,
+            block_squeeze_excitation=False,
+            block_squeeze_excitation_config={},
             residual=True,
             residual_act=True,
             residual_norm=None,
@@ -381,6 +383,8 @@ class Stack(keras.Model):
                 ratio=squeeze_excitation_ratio)
         elif channel_squeeze_excitation:
             self.se=ChannelSqueezeExcitation(filters)
+        elif block_squeeze_excitation:
+            self.se=BlockSqueezeExcitation(filters,**block_squeeze_excitation_config)
         else:
             self.se=False
         self.residual=self._residual(
@@ -388,10 +392,14 @@ class Stack(keras.Model):
             residual_act,
             residual_norm,
             residual_norm_config,
-            norm,
-            norm_config,
             self.filters_list[-1],
             output_stride)
+        if self.residual and padding=='valid':
+            self.res_crop=int(kernel_size//2)*len(self.filters_list)
+            if (kernel_size/2)==self.res_crop:
+                raise ValueError('Residuals with valid padding must have odd kernel_size')
+        else:
+            self.res_crop=False
         self.stack=self._build_stack(
             output_stride,
             input_dilation_rate,
@@ -400,10 +408,12 @@ class Stack(keras.Model):
 
 
     def __call__(self,x,training=False,**kwargs):
-        if self.residual==Stack.IDENTITY:
+        if self.residual:
             res=x
-        elif self.residual:
-            res=self.residual(x)
+            if self.residual!=Stack.IDENTITY:
+                if self.res_crop:
+                    res=res[:,self.res_crop:-self.res_crop,self.res_crop:-self.res_crop,:]
+                res=self.residual(res)
         for layer in self.stack:
             x=layer(x)
         if self.se:
@@ -485,8 +495,6 @@ class Stack(keras.Model):
             residual,
             residual_act,
             residual_norm,
-            residual_config,
-            norm,
             residual_norm_config,
             filters,
             output_stride):
@@ -496,9 +504,9 @@ class Stack(keras.Model):
             else:
                 act=self.act
             if residual_norm is None:
-                residual_norm=norm
+                residual_norm=self.norm
             if residual_norm_config is None:
-                residual_norm_config=norm_config
+                residual_norm_config=self.norm_config
             cfig=self.shared_config.copy()
             cfig['dropout']=False
             residual=Conv(
@@ -762,6 +770,51 @@ class SqueezeExcitation(keras.Model):
         return layers.multiply([x,y])
 
 
+class BlockSqueezeExcitation(keras.Model):
+    """ BlockSqueezeExcitation
+    """
+
+    def __init__(self,filters,ratio=16,pool_type='avg',size=None,grid_size=3):
+        super(BlockSqueezeExcitation, self).__init__()
+        if pool_type in ['avg','max']:
+            self.pool_type=pool_type
+        else:
+            raise ValueError('BlockSqueezeExcitation: pooling must be max or avg')
+        self.grid_size=grid_size
+        self._set_pool_up(size)     
+        self.reduce=layers.Dense(filters//ratio,activation='relu')
+        self.expand=layers.Dense(filters,activation='sigmoid')
+
+
+    def _get_pool_layer(self):
+        if self.pool_type=='avg':
+            return layers.AveragePooling2D
+        elif self.pool_type=='max':
+            return layers.MaxPool2D
+ 
+
+    def _set_pool_up(self,size):
+        if size:
+            _ps=size/self.grid_size
+            self.pool_size=int(_ps)
+            if _ps==self.pool_size:
+                self.pooling=self._get_pool_layer()(self.pool_size,padding='same')
+                self.up=layers.UpSampling2D(self.pool_size)
+            else:
+                raise ValueError('BlockSqueezeExcitation: grid_size/size must an integer')     
+        else:
+            self.pool_size=None
+
+
+    def __call__(self,x,training=False):
+        if not self.pool_size:
+            self._set_pool_up(x.shape[1])  
+        y=self.pooling(x)
+        y=self.reduce(y)
+        y=self.expand(y)
+        y=self.up(y)
+        return layers.multiply([x,y])
+
 
 class ChannelSqueezeExcitation(keras.Model):
     """ ChannelSqueezeExcitation
@@ -909,6 +962,7 @@ class SegmentClassifier(keras.Model):
             bias_class_weights=None,
             name=None,
             named_layers=True,
+            padding='same',
             **stack_config):
         super(SegmentClassifier, self).__init__()
         self.block_name=name or self.name
@@ -926,6 +980,7 @@ class SegmentClassifier(keras.Model):
             self.preclassifier=Stack(
                 filters_list=filters_list[:-1],
                 kernel_size_list=kernel_size_list[:-1],
+                padding=padding,
                 seperable=seperable_preclassification,
                 residual=residual_preclassification,
                 name=self._layer_name('stack'),
@@ -940,7 +995,7 @@ class SegmentClassifier(keras.Model):
                     bias_class_weights,
                     dtype=tf.dtypes.float32)
                 bias_class_weights=bias_class_weights/tf.reduce_sum(bias_class_weights)
-                bias_class_weights=tf.math.log(bias_class_weights)
+                bias_class_weights=tf.math.log(bias_class_weights+EPS)
                 print('FIXED BIAS INITIALIZER:',_,tf.nn.softmax(bias_class_weights))
                 bias_initializer=get_fixed_initializer(bias_class_weights)
             else:
@@ -948,6 +1003,7 @@ class SegmentClassifier(keras.Model):
             self.classifier=Conv(
                     filters=filters_list[-1],
                     kernel_size=kernel_size_list[-1],
+                    padding=padding,
                     norm=output_norm,
                     norm_config=output_norm_config,
                     act=False,
